/* ========================================================================== */
/* == UPMC/master/info/4I506 -- Janvier 2016/2017/2018                     == */
/* == SU/FSI/master/info/MU4IN503 -- Janvier 2020/2021/2022                == */
/* == Analyse des programmes et sémantiques                                == */
/* ========================================================================== */
/* == hello-APS Syntaxe C                                                  == */
/* == Fichier: parser.y                                                    == */
/* == Grammaire et génération terme Prolog                                 == */
/* == Nota: prend son entrée sur stdin                                     == */
/* ========================================================================== */

%{

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <math.h>

#include "ast.h"
#include "prologTerm.h"

int yylex (void);
int yyerror (char *);

 Cmds theProg;
 
%}

%token<num>  NUM
%token<str>  IDENT
%token       ECHO
%token       LPAR RPAR
%token       LBRA RBRA

%union {
  int num;
  char* str;
  Expr expr;
  Exprs exprs;
  Stat stat;
  Cmds cmds;
  Cmds prog;
}

%type<expr> expr
%type<exprs> exprs
%type<stat> stat
%type<cmds> cmds
%type<cmds> prog

%start prog
%%

prog: LBRA cmds RBRA    { theProg = $2; }
;

cmds:
stat  { $$ = addStat($1,NULL); }
;

stat:
ECHO expr  { $$ = newASTEcho($2); }

expr:
NUM                       { $$ = newASTNum($1); }
| IDENT                     { $$ = newASTId($1); }
| LPAR expr exprs RPAR      { $$ = newASTApp($2,$3); }
;
exprs:
  expr       { $$ = addExpr($1,NULL); }
| expr exprs { $$ = addExpr($1,$2); }
%%

int yyerror(char *s) {
  printf("error: %s\n",s);
  return 1;
}

int main(int argc, char **argv) {
  if (argc < 2) {
    fprintf(stderr, "usage: ./prologTerm <fichier.aps>\n");
    return 1;
  }
  FILE* infile = freopen(argv[1], "r", stdin);
  yyparse();
  fclose(infile);
  printProg(theProg);
  printf(".\n");
  return 0;
}

